#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)

"""
Script: Generate a list of QAR ini configuration files
---
- Read from download lists
- Generate ini file to be used by the
"""

import os
import json
import glob
import requests

def generate_ini(rootd, pattern, variable_e, datatype, variable, res, ts, origin, system, dataset, is_seasonal, mf, mv, dr):
	input_dir = ''.format(ds=dataset, v=variable)

	ini='''[general]
input = /shared/cds_downloads/{r}/{ds}/{v}
fpattern = {p}
log_dir = /shared/QAR_dqc/logs 
res_dir = /shared/QAR_dqc/results
forms_dir = /shared/cds-forms-c3s

[dataset]
variable = {ve}
datatype = {dt}
cds_dataset = {ds}
cds_variable = {v}

[file_format]

[standard_compliance]

[spatial_completeness]
mask_file = {mf}
mask_var = {mv}

[temporal_completeness]
origin = {o}
system = {s}
is_seasonal = {is_s}

[spatial_consistency]
grid_interval = {res}
grid_type = lonlat
dual_resolution = {dr}

[temporal_consistency]
time_step = {t}
time_granularity = month

[valid_ranges]
valid_min =
valid_max =	
'''.format(
	p=pattern, ve=variable_e, dt=datatype, v=variable, is_s=is_seasonal, dr=dr,
	o=origin, s=system, ds=dataset, t=ts, r=rootd, res=res, mf=mf, mv=mv)

	return ini

"""
Get mask file (if needed)
"""
def get_mask_file(root, var, ds, o, s, mask_variables):
	m = ''
	mv = ''

	if var in mask_variables:

		# SEASONAL: Only original resolution has a variable land_sea_mask
		if root == 'seasonal' and 'original' in ds:
			m='/shared/cds_downloads/masks/{d}-{o}-{s}-land_sea_mask.grib'.format(d=ds, o=o, s=s)
			mv='lsm'
		
		# REANALYSIS: Only single-levels
		if root == 'reanalysis' and 'single-levels' in ds:
			m='/shared/cds_downloads/masks/{d}-monthly_averaged_reanalysis-land_sea_mask.grib'.format(d=ds)
			mv='lsm'

	return m, mv

"""
Given a cds variable name (long-name)
Return a parameter identifier in the mapping.json
"""
def get_qar_names(jsonf, codes_var, mask_variables):
	qar_names = []
	inis = []

	with open(jsonf, 'r') as f:
		dd=json.load(f)
		
		for dl in dd:
			for v in dl['variables']:				

				d = dl['dataset']
				ve = codes_var[dl['dataset']]['variables'][v]	

				# Seasonal
				if 'originating_centre' in dl['form']:
					root = 'seasonal'

					# Grid resolution
					res = 1
					dr = 'no'

					# Product type
					dt = 'fcmean, hcmean, em'

					# Origin/System
					o = dl['form']['originating_centre']
					s = dl['form']['system']					
					
					# File pattern
					patt = o+'-'+s+'*.grib'
					
					# Only original has land_sea_mask
					mf,mv = get_mask_file(root, v, d, o, s, mask_variables)

					qar_name = d+'-'+v+'-'+o+'-'+s+'.ini'
					ini = generate_ini(root, patt, ve, dt, v, res, 1, o, s, d, 'monthly', mf, mv, dr)
						
				# Reanalysis
				else:	
					root = 'reanalysis'	

					# Grid resolution (two resolutions allowed) - 0.25-0.5 and 0.5-1.0
					res = 0.5
					dr = 'yes'

					# File pattern
					patt = 'monthly_averaged_reanalysis-*.grib'

					# Product type
					dt = 'an, fc'
					
					mf,mv = get_mask_file(root, v, d, '', '', mask_variables)
					qar_name = d+'-'+v+'.ini'	
					ini = generate_ini(root, patt, ve, dt, v, res, 1, '', '', d, 'no', mf, mv, dr)

				# Next
				qar_names.append(qar_name)
				inis.append(ini)

	return zip(qar_names, inis)

def write_file(outf, string):
	with open(outf, 'w') as f:
		f.write(string)

def main():

	# Config paths
	conf_dir="/data/joan/c3s512-wp1-toolbox/conf"
	qar_dir="/data/joan/c3s512-wp1-datachecker/dqc_wrapper/QAR_dqc/conf"
	datasets = "/data/joan/c3s512-wp1-datachecker/dqc_wrapper/grib/datasets.json"

	# Output directory
	out_dir="/tmp/conf_generate"
	if not os.path.exists(out_dir):
		os.makedirs(out_dir)

	# Get codes
	codes_var = json.load(open(datasets,'r'))

	# Variables that require a land_sea_mask
	mask_variables = []
	with open('./mask_variables') as f:
		mask_variables = f.read().splitlines()

	# For downloaded datasets
	for json_conf in glob.glob(conf_dir+'/*monthly*json'):

		# Get file names
		qar_names = get_qar_names(json_conf, codes_var, mask_variables)
		for data in qar_names:
			q,i = data
			qar=os.path.join(qar_dir, q)
			qar_out=os.path.join(out_dir, q)
			
			if os.path.exists(qar_out):
				print('Skipping ' + q)				
			else:
				print('Writting ' + q)
				write_file(qar_out, i)

			if os.path.exists(qar):
				os.system('diff -b {} {}'.format(qar_out, qar))

if __name__ == "__main__":
    main()
